@@ -174,6 +174,12 @@ class MobileService:
         """
         Smart swipe from element with automatic start position calculation
 
+        Start position is adjusted so finger movement feels natural:
+        - up: start from bottom (3/4 height) → swipe upward
+        - down: start from top (1/4 height) → swipe downward
+        - left: start from right (3/4 width) → swipe leftward
+        - right: start from left (1/4 width) → swipe rightward
+
         Args:
             element: Element to swipe from
             direction: Swipe direction (up, down, left, right)
@@ -182,26 +188,44 @@ class MobileService:
         start_y = element.center_y
 
         # Adjust start position based on direction for better UX
-        if direction == "down":
+        if direction == "up":
+            # Start from bottom of element to swipe UP
             start_y = element.y + element.height * 3 // 4
-        elif direction == "up":
+        elif direction == "down":
+            # Start from top of element to swipe DOWN
             start_y = element.y + element.height // 4
-        elif direction == "right":
-            start_x = element.x + element.width // 4
         elif direction == "left":
+            # Start from right of element to swipe LEFT
             start_x = element.x + element.width * 3 // 4
+        elif direction == "right":
+            # Start from left of element to swipe RIGHT
+            start_x = element.x + element.width // 4
 
         await self.client.swipe(direction=direction, start_x=start_x, start_y=start_y)
 
     async def scroll_element(self, element: ActiveElement, direction: str) -> None:
         """
-        Smart scroll on element by delegating to swipe_element for consistent behavior
+        Smart scroll on element by delegating to swipe_element with inverted direction
+
+        SCROLL semantics: direction = where content moves (what you want to see)
+        - scroll down = see content below = finger swipes UP
+        - scroll up = see content above = finger swipes DOWN
+        - scroll left = see content on left = finger swipes RIGHT
+        - scroll right = see content on right = finger swipes LEFT
 
         Args:
             element: Element to scroll
-            direction: Scroll direction (up, down, left, right)
+            direction: Scroll direction (up, down, left, right) - content movement
         """
-        await self.swipe_element(element, direction)
+        # Invert direction: scroll down = swipe up, scroll up = swipe down
+        direction_mapping = {
+            "up": "down",  # scroll up (see above) = swipe finger down
+            "down": "up",  # scroll down (see below) = swipe finger up
+            "left": "right",  # scroll left (see left) = swipe finger right
+            "right": "left",  # scroll right (see right) = swipe finger left
+        }
+        swipe_direction = direction_mapping.get(direction, direction)
+        await self.swipe_element(element, swipe_direction)
 
     def _resize_image(self, image: Image.Image, max_size: int) -> Image.Image:
         """Resize image so that minimum side is max_size"""
