@@ -6,7 +6,6 @@ import sentry_sdk
 from pydantic import ConfigDict, Field
 from pydantic_settings import BaseSettings
 from sentry_sdk.integrations.asyncio import AsyncioIntegration
-from sentry_sdk.integrations.fastapi import FastApiIntegration
 
 
 class NoqaSettings(BaseSettings):
@@ -30,6 +29,9 @@ class RunnerSettings(NoqaSettings):
         default="http://localhost:4723",
         description="Default Appium URL for the agent API",
     )
+    MAX_STEPS: int = Field(
+        default=100, gt=0, description="Maximum number of steps for test execution"
+    )
 
 
 settings = RunnerSettings()
@@ -42,10 +44,18 @@ def sentry_init(
     if not dsn:
         return
 
+    integrations = [AsyncioIntegration()]
+    try:
+        from sentry_sdk.integrations.fastapi import FastApiIntegration
+
+        integrations.append(FastApiIntegration())
+    except ImportError:
+        pass
+
     sentry_sdk.init(
         dsn=dsn,
         environment=environment,
         traces_sample_rate=0.1,
         enable_logs=enable_logs,
-        integrations=[FastApiIntegration(), AsyncioIntegration()],
+        integrations=integrations,
     )
