@@ -11,6 +11,7 @@ from noqa_runner.domain.models.actions.open_url import OpenUrl
 from noqa_runner.domain.models.actions.restart_app import RestartApp
 from noqa_runner.domain.models.actions.scroll import Scroll
 from noqa_runner.domain.models.actions.stop import Stop
+from noqa_runner.domain.models.actions.swipe import Swipe
 from noqa_runner.domain.models.actions.tap import Tap
 from noqa_runner.domain.models.actions.terminate_app import TerminateApp
 from noqa_runner.domain.models.actions.wait import Wait
@@ -23,7 +24,7 @@ class ActionData(BaseModel):
     action: Union[
         Tap,
         InputText,
-        # Swipe,
+        Swipe,
         Scroll,
         Wait,
         Stop,
