@@ -18,6 +18,10 @@ from noqa_runner.utils.retry_decorator import with_retry
 logger = get_logger(__name__)
 
 
+def get_appstore_url(app_store_id: str, country_code: str = "us") -> str:
+    return f"https://apps.apple.com/{country_code}/app/{app_store_id}"
+
+
 def handle_appium_errors(func):
     """Decorator to catch Appium errors and convert them to AppiumError"""
 
@@ -161,7 +165,14 @@ class AppiumClient:
 
     @handle_appium_errors
     async def swipe(self, direction: str, start_x: int, start_y: int) -> None:
-        """Swipe from element center, swipe distance is always half of screen size"""
+        """Swipe from element center, swipe distance is always half of screen size
+
+        Direction is FINGER MOVEMENT:
+        - up: finger moves up (from bottom to top)
+        - down: finger moves down (from top to bottom)
+        - left: finger moves left (from right to left)
+        - right: finger moves right (from left to right)
+        """
         # Get screen dimensions
         screen_width = self.resolution["width"]
         screen_height = self.resolution["height"]
@@ -170,22 +181,22 @@ class AppiumClient:
         swipe_distance_vertical = screen_height // 2
         swipe_distance_horizontal = screen_width // 2
 
-        if direction == "down":
-            # Swipe up (from start to top)
+        if direction == "up":
+            # Swipe finger UP (move from bottom to top)
             end_x = start_x
             end_y = max(0, start_y - swipe_distance_vertical)
-        elif direction == "up":
-            # Swipe down (from start to bottom)
+        elif direction == "down":
+            # Swipe finger DOWN (move from top to bottom)
             end_x = start_x
             end_y = min(screen_height, start_y + swipe_distance_vertical)
-        elif direction == "right":
-            # Swipe right (from start to right)
-            end_x = min(screen_width, start_x + swipe_distance_horizontal)
-            end_y = start_y
         elif direction == "left":
-            # Swipe left (from start to left)
+            # Swipe finger LEFT (move from right to left)
             end_x = max(0, start_x - swipe_distance_horizontal)
             end_y = start_y
+        elif direction == "right":
+            # Swipe finger RIGHT (move from left to right)
+            end_x = min(screen_width, start_x + swipe_distance_horizontal)
+            end_y = start_y
         else:
             raise ValueError(f"Invalid swipe direction: {direction}")
 
@@ -198,14 +209,12 @@ class AppiumClient:
         """Navigate to URL"""
         await self.loop.run_in_executor(None, lambda: self.driver.get(url))
 
-    @handle_appium_errors
     async def accept_system_alert(self) -> None:
         """Accept system alert"""
         await self.loop.run_in_executor(
             None, lambda: self.driver.switch_to.alert.accept()
         )
 
-    @handle_appium_errors
     async def dismiss_system_alert(self) -> None:
         """Dismiss system alert"""
         await self.loop.run_in_executor(
@@ -316,6 +325,8 @@ class AppiumClient:
         await self.activate_app("com.apple.TestFlight")
         logger.info("TestFlight activated")
 
+        initial_button_name = None
+
         # Wait for app to be installed
         max_wait_time = 20
         wait_start = 0
@@ -324,16 +335,6 @@ class AppiumClient:
             wait_start += 1
 
             try:
-                app_state = await self.query_app_state(app_bundle_id)
-                if app_state == 4:
-                    logger.info(
-                        "App installed and running",
-                        bundle_id=app_bundle_id,
-                        app_state=app_state,
-                    )
-                    return True
-
-                # Get current page source
                 xml_str = await self.get_page_source()
 
                 # Handle alerts (e.g., "Do you want to install this app?")
@@ -354,8 +355,16 @@ class AppiumClient:
                         main_button.get_attribute("name")
                         and len(main_button.get_attribute("name")) > 2
                     ):
+                        if not initial_button_name:
+                            initial_button_name = main_button.get_attribute("name")
+                        else:
+                            await self.terminate_app("com.apple.TestFlight")
+                            await asyncio.sleep(1)
+                            await self.activate_app(app_bundle_id)
+                            return True
                         main_button.click()
                         continue
+
             except Exception:
                 pass
 
@@ -366,6 +375,73 @@ class AppiumClient:
         )
         return False
 
+    @handle_appium_errors
+    @with_retry(max_attempts=3)
+    async def install_app_appstore(
+        self, app_bundle_id: str, app_store_id: str | None = None
+    ) -> bool | None:
+        # Remove existing app
+        await self.remove_app(app_bundle_id)
+        logger.info("Existing app removed", bundle_id=app_bundle_id)
+
+        max_wait_time = 20
+        wait_start = 0
+        while wait_start < max_wait_time:
+            await asyncio.sleep(1)
+            wait_start += 1
+
+            try:
+                app_state = await self.query_app_state(app_bundle_id)
+                if app_state == 4:
+                    logger.info(
+                        "App installed and running",
+                        bundle_id=app_bundle_id,
+                        app_state=app_state,
+                    )
+                    return True
+
+                appstore_state = await self.query_app_state("com.apple.AppStore")
+                if appstore_state != 4:
+                    self.driver.get(get_appstore_url(app_store_id=app_store_id))
+                    continue
+
+                # Get current page source
+                xml_str = await self.get_page_source()
+
+                if "payment-sheet" in xml_str:
+                    buttons = await self.get_elements(
+                        "//XCUIElementTypeOther[@name='payment-sheet']//XCUIElementTypeButton"
+                    )
+                    if buttons:
+                        buttons[-1].click()
+                        continue
+
+                if "AppStore.offerButton" in xml_str:
+                    if (
+                        "AppStore.offerButton[state=downloading]" in xml_str
+                        or "AppStore.offerButton[state=loading]" in xml_str
+                    ):
+                        logger.info("App is downloading, waiting...")
+                        continue
+                    elif "AppStore.offerButton[state=open]" in xml_str:
+                        open_button = await self.get_element(
+                            "//XCUIElementTypeCell[contains(@name, 'AppStore.shelfItem.productTopLockup')]//XCUIElementTypeButton[@name='AppStore.offerButton[state=open]']"
+                        )
+                        if open_button:
+                            await self.terminate_app("com.apple.AppStore")
+                            await asyncio.sleep(1)
+                            await self.activate_app(app_bundle_id)
+                            continue
+                    elif "AppStore.offerButton[state=redownload]" in xml_str:
+                        main_button = await self.get_element(
+                            "//XCUIElementTypeCell[contains(@name, 'AppStore.shelfItem.productTopLockup')]//XCUIElementTypeButton[@name='AppStore.offerButton[state=redownload]']"
+                        )
+                        if main_button:
+                            main_button.click()
+                            continue
+            except Exception as e:
+                logger.info("Transient error during installation check", error=str(e))
+
     async def close(self) -> None:
         """Close the Appium driver connection"""
         if self.driver and hasattr(self.driver, "quit"):
